
#include <proto/exec.h>
#include <proto/dos.h>
#include <proto/graphics.h>
#include <proto/intuition.h>
#include <string.h>
#include <newmouse.h>

#ifndef IDCMP_EXTENDEDMOUSE

	/* for 68k programs running on OS 4.x */

	#define IDCMP_EXTENDEDMOUSE 0x08000000L

	struct IntuiWheelData
	{
    UWORD Version;
    UWORD Reserved;
    WORD  WheelX;
    WORD  WheelY;
	};

	#define IMSGCODE_INTUIWHEELDATA (1<<15)

#endif

void check_wheel (struct Window *win,UWORD code)

{
char *text;

switch (code)
	{
case NM_WHEEL_UP:
	text = "wheel turned up";
	break;
case NM_WHEEL_DOWN:
	text = "wheel turned down";
	break;
case NM_WHEEL_LEFT:
	text = "wheel turned left";
	break;
case NM_WHEEL_RIGHT:
	text = "wheel turned right";
	break;
default:
	text = NULL;
	}

if (text)
	{
	struct RastPort *rp = win->RPort;
	long x0 = win->BorderLeft + 8;
	long y0 = win->BorderTop + 8;
	long x1 = x0 + win->GZZWidth - 17;
	long y1 = y0 + rp->TxHeight - 1;

	SetABPenDrMd (rp,1,0,JAM1);
	EraseRect (rp,x0,y0,x1,y1);
	Move (rp,x0,y0+rp->TxBaseline);
	Text (rp,text,strlen(text));
	}
}

int main (void)
{
struct Window *win;
struct IntuiMessage *imsg;
BOOL cont;

if (win = OpenWindowTags (NULL,
		WA_Left,80,
		WA_Top,80,
		WA_Width,480,
		WA_Height,80,
		WA_Title,"Please turn the mouse wheel(s)",
		WA_Flags,WFLG_CLOSEGADGET | WFLG_DRAGBAR | WFLG_DEPTHGADGET | WFLG_ACTIVATE | WFLG_NOCAREREFRESH,
		WA_IDCMP,IDCMP_CLOSEWINDOW | IDCMP_RAWKEY | IDCMP_EXTENDEDMOUSE,
		TAG_END))
	{
	cont = TRUE;
	do	{
		if (Wait ((1L << win->UserPort->mp_SigBit) | SIGBREAKF_CTRL_C) & SIGBREAKF_CTRL_C)
			cont = FALSE;

		while (imsg = (struct IntuiMessage *) GetMsg (win->UserPort))
			{
			switch (imsg->Class)
				{
			case IDCMP_RAWKEY:
				if (imsg->Code == 0x45) /* Esc */
					cont = FALSE;
				else
					check_wheel (win,imsg->Code);
				break;
			case IDCMP_EXTENDEDMOUSE:
				if (imsg->Code == IMSGCODE_INTUIWHEELDATA)
					{
					struct IntuiWheelData *wd = imsg->IAddress;
					UWORD code;
					if (wd->WheelY > 0)
						code = NM_WHEEL_DOWN;
					else if (wd->WheelY < 0)
						code = NM_WHEEL_UP;
					else if (wd->WheelX > 0)
						code = NM_WHEEL_RIGHT;
					else if (wd->WheelX < 0)
						code = NM_WHEEL_LEFT;
					else
						code = 0;
					check_wheel (win,code);
					}
				break;
			case IDCMP_CLOSEWINDOW:
				cont = FALSE;
				break;
				}
			ReplyMsg ((struct Message *)imsg);
			}
		}
	while (cont);

	CloseWindow (win);
	}

return (0);
}
